﻿using System;
using System.Collections.Specialized;
using System.Web;

namespace MvcFakes
{

    public class FakeHttpRequest : HttpRequestBase
    {
        private readonly Uri _url;
        private readonly string _appRelativeUrl;
        private readonly NameValueCollection _formParams;
        private readonly NameValueCollection _queryStringParams;
        private readonly HttpCookieCollection _cookies;
        private readonly string _httpMethod;
        private readonly bool _isAuthenticated = false;

        public FakeHttpRequest(Uri url, string appRelativeUrl, string httpMethod, bool isAuthenticated, NameValueCollection formParams, NameValueCollection queryStringParams, HttpCookieCollection cookies)
        {

            if (!appRelativeUrl.StartsWith("~"))
                throw new Exception("appRelativeUrl must start with ~");

            _url = url;
            _appRelativeUrl = appRelativeUrl;
            _httpMethod = httpMethod;
            _isAuthenticated = isAuthenticated;
            _formParams = formParams;
            _queryStringParams = queryStringParams;
            _cookies = cookies;
        }

        public override NameValueCollection Form
        {
            get
            {
                return _formParams;
            }
        }

        public override NameValueCollection QueryString
        {
            get
            {
                return _queryStringParams;
            }
        }

        public override HttpCookieCollection Cookies
        {
            get
            {
                return _cookies;
            }
        }

        public override string AppRelativeCurrentExecutionFilePath
        {
            get { return _appRelativeUrl; }
        }


        public override string HttpMethod
        {
            get
            {
                return _httpMethod;
            }
        }

        public override Uri Url
        {
            get
            {
                return _url;
            }
        }

        public override string PathInfo
        {
            get
            {
                return String.Empty;
            }
        }


        public override bool IsLocal
        {
            get
            {
                return (Url.Host.ToLower() ==  "localhost" || Url.Host == "127.0.01");
            }
        }

        public override bool IsAuthenticated
        {
            get
            {
                return _isAuthenticated;
            }
        }


    }



}
