﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Web.Mvc;
using System.Web.Routing;
using MvcFakes;

namespace RouteDebugger.Controllers
{
    public class RouteDebuggerController : Controller
    {
        public string Index(string url, string httpMethod)
        {

            url = MakeAppRelative(url);
            httpMethod = httpMethod ?? "GET";

            var fakeContext = new FakeHttpContext(url, httpMethod);
            var httpMethodOptions = formatOptions(httpMethod, new string[]{"GET","POST","PUT","DELETE","HEAD"} );
            var routeDataText = GetRoutesText(fakeContext);
            return string.Format(htmlFormat, url, httpMethodOptions, routeDataText);
        }

        private string GetRoutesText(FakeHttpContext fakeContext)
        {
            var sb = new StringBuilder();
            foreach (NamedRoute route in RouteTable.Routes)
            {
                var rd = route.GetRouteData(fakeContext);
                // Get match
                var isMatch = false;
                var match = rd == null ? "No Match" : "Match";

                // Get values
                var values = "N/A";
                if (rd != null)
                {
                    isMatch = true;
                    values = formatValues(rd.Values);
                }

                // Get defaults
                var defaults = formatValues(route.Defaults);

                // Get constraints
                var constraints = formatValues(route.Constraints);

                // Get dataTokens
                var dataTokens = formatValues(route.DataTokens);

                // Create table row
                var row = formatRow(isMatch, match, route.Name, route.Url, defaults, constraints, dataTokens, values);
                sb.Append(row);
            }
            return sb.ToString();
        }

        private string formatValues(RouteValueDictionary values)
        {
            if (values == null)
                return "N/A";
            var col = new List<String>();
            foreach (string key in values.Keys)
            {
                object value = values[key] ?? "[null]";
                col.Add(key + "=" + value.ToString());
            }
            return String.Join(", ", col.ToArray());
        }

        private string formatOptions(string selected, string[] values)
        {
            var sb = new StringBuilder();
            foreach (string value in values)
            {
                var showSelected = String.Empty;
                if (value == selected)
                    showSelected = "selected='selected'";
                sb.AppendFormat("<option value='{0}' {1}>{0}</option>", value, showSelected);
            }
            return sb.ToString();
        }


        private string formatRow(bool hilite, params string[] cells)
        {
            var sb = new StringBuilder();
            sb.Append(hilite ? "<tr class='hilite'>":"<tr>");
            foreach (string cell in cells)
                sb.AppendFormat("<td>{0}</td>", cell);
            sb.Append("</tr>");
            return sb.ToString();
        }

        private string MakeAppRelative(string url)
        {
            if (!url.StartsWith("~"))
            {
                if (!url.StartsWith("/"))
                    url = "~/" + url;
                else
                    url = "~" + url;
            }
            return url;
        }


        const string htmlFormat = @"
            <html>
            <head>
                <title>Route Debugger</title>
                <style type='text/css'>
                table {{ border-collapse:collapse }}
                td
                {{
                    font:10pt Arial;
                    border: solid 1px black;
                    padding:3px; 
                }}
                .hilite {{background-color:lightgreen}}
                </style>
            </head>
            <body>
            
            <form action=''>
            <label for='url'>URL:</label>
            <input name='url' size='60' value='{0}' />
            <select name='httpMethod'>
            {1}
            </select>
            <input type='submit' value='Debug' />
            </form>

            <table>
            <caption>Routes</caption>
            <tr>
                <th>Matches</th>
                <th>Name</th>
                <th>Url</th>
                <th>Defaults</th>
                <th>Constraints</th>
                <th>DataTokens</th>
                <th>Values</th>
            </tr>
            {2}
            </table>

            </body>
            </html>

            ";

    }
}


