﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Web.Mvc;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Tip30;
using Tip30.Controllers;
using MvcFakes;
using System.Web.Routing;
using RouteDebugger;
using System.Security;
using System.Security.Principal;


namespace Tip30Tests.Routes
{
    [TestClass]
    public class RouteTests
    {


        [TestMethod]
        public void TestInsertIsPost()
        {
            // Arrange
            var routes = new RouteCollection();
            GlobalApplication.RegisterRoutes(routes);

            // Act with POST request
            var fakeContext1 = new FakeHttpContext("~/Product/Insert", "POST");
            var routeData1 = routes.GetRouteData(fakeContext1);

            // Assert
            NamedRoute matchedRoute1 = (NamedRoute)routeData1.Route;
            Assert.AreEqual("Product", matchedRoute1.Name);

            // Act with GET request
            var fakeContext2 = new FakeHttpContext("~/Product/Insert");
            var routeData2 = routes.GetRouteData(fakeContext2);

            // Assert
            NamedRoute matchedRoute2 = (NamedRoute)routeData2.Route;
            Assert.AreNotEqual("Product", matchedRoute2.Name);
        }




        [TestMethod]
        public void TestAdminRouteIsAuthenticated()
        {
            // Arrange
            var routes = new RouteCollection();
            GlobalApplication.RegisterRoutes(routes);

            // Act with authenticated request
            var fakeUser = new FakePrincipal(new FakeIdentity("Bob"), null);
            var fakeContext1 = new FakeHttpContext(new Uri("http://localhost/Admin/Index"), "~/Admin/Index", fakeUser);
            var routeData1 = routes.GetRouteData(fakeContext1);

            // Assert
            NamedRoute matchedRoute1 = (NamedRoute)routeData1.Route;
            Assert.AreEqual("Admin", matchedRoute1.Name);

            // Act with anonymous request
            var fakeContext2 = new FakeHttpContext(new Uri("http://localhost/Admin/Index"), "~/Admin/Index");
            var routeData2 = routes.GetRouteData(fakeContext2);

            // Assert
            Assert.IsNull(routeData2);
        }

        
        
        
        [TestMethod]
        public void TestAdminRouteIsLocal()
        {
            // Arrange
            var routes = new RouteCollection();
            GlobalApplication.RegisterRoutes(routes);
            var fakeUser = new FakePrincipal(new FakeIdentity("Bob"), null);
            
            // Act with local request
            var fakeContext1 = new FakeHttpContext(new Uri("http://localhost/Admin/Index"), "~/Admin/Index", fakeUser);
            var routeData1 = routes.GetRouteData(fakeContext1);

            // Assert
            NamedRoute matchedRoute1 = (NamedRoute)routeData1.Route;
            Assert.AreEqual("Admin", matchedRoute1.Name);

            // Act with remote request
            var fakeContext2 = new FakeHttpContext(new Uri("http://microsoft.com/Admin/Index"), "~/Admin/Index", fakeUser);
            var routeData2 = routes.GetRouteData(fakeContext2);

            // Assert
            Assert.IsNull(routeData2);
        }





    }
}
